//
//  MKVI Monorail Demo
//
//  This is a test script to control multiple trains stopping at each
//  side of the station during a complete circuit of the beamway.  The
//  script contains examples of simple message handling for multiple
//  trains simultaneously, controlled animation, sound effects, various
//  camera options, and the simple interactions between trains needed to
//  keep them running indeffinately.  It does not yet contain any switching
//  or signaling.
//
//  Note that the structure of this scenario is heavily based on the original
//  scenario, and may not use many features that are now available in the
//  Trainz Script lanugage.  Future versions of this and other scenarios
//  will use such features as appropriate, but his ones remains pretty old
//  fashioned.
//
//  Copyright (C) 2002 Jack Emmerichs, Milwaukee Wisconsin, 2002.
//      Users may modify this scenario or use it as a basis for their own
//      work, but should not redistribute it without including this original
//      copyright notice.
//
//  This is the version developed for the Ultimate Trainz Collection (UTC),
//  and may not run on older builds of the Trainz program that does not support
//  the Scenario class.  See the scenario named MKVIDemo.gs for older systems.


include "trainz.gs"
include "common.gs"
include "world1.gs"
//include "stringtable.gs"

//===========================================================================//
class TrainData
//
//   Class:      TrainData
//   Purpose:    This is a class to hold train informaiton that we cannot hold
//                  in Train objects.  If we use an array of trains, we can also
//                  use an array of TrainData objects to describe them.  At some
//                  point, we should be able to store this object within each train.
//   Copyright (C) 2002 Jack Emmerichs, Milwaukee Wisconsin, 2002.
//   Created by: Jack Emmerichs
//
//   Change History---------
//      06/25/02    Started.

{
//  Constant Values:  NONE

//  Protected Instance Attributes

    int     m_nID            = 0;       // the ID value of this train
    bool    m_bInStation     = false;   // this train is currently in a station flag
    string  m_strName        = "";      // the name of this train
    string  m_strLastStation = "";      // the name of the last station visited by this train

//---------------------------------------------------------------------------//
//  Value setting routines

	public void SetID           (int nID)
    	{m_nID            = nID;}
	public void SetIsInStation  (bool bInStation)
    	{m_bInStation     = bInStation;}
	public void SetName         (string strName)
    	{m_strName        = strName;}
	public void SetLastStation  (string strLastStation)
    	{m_strLastStation = strLastStation;}

//---------------------------------------------------------------------------//
//  Value getting routines

	public int      GetID ()
    	{return m_nID;}
	public bool     GetIsInStation ()
    	{return m_bInStation;}
	public string   GetName ()
    	{return m_strName;}
	public string   GetLastStation ()
    	{return m_strLastStation;}
};
//  End of class TrainData
//===========================================================================//



//===========================================================================//
class StationData
//
//   Class:      StationData
//   Purpose:    This is a class to hold station informaiton that we cannot hold
//                  in any other objects.  With only one station in the Demo,
//                  we need only one of these objects.
//   Copyright (C) 2002 Jack Emmerichs, Milwaukee Wisconsin, 2002.
//   Created by: Jack Emmerichs
//
//   Change History---------
//      09/01/02    Started.

{
//  Constant Values:  NONE

//  Protected Instance Attributes

    int     m_nTrainsInStation = 0;     // the number of trains in the station
    int     m_nFirstTrainIn    = 0;     // the index of the first train to arrive
    int     m_nSecondTrainIn   = 0;     // the index of the second train to arrive

//---------------------------------------------------------------------------//
//  Value setting routines

	public void SetTrainsInStation  (int nTrainsInStation)
    	{m_nTrainsInStation     = nTrainsInStation;}
	public void SetFirstTrainIn     (int nFirstTrainIn)
    	{m_nFirstTrainIn        = nFirstTrainIn;}
	public void SetSecondTrainIn     (int nSecondTrainIn)
    	{m_nSecondTrainIn       = nSecondTrainIn;}

//---------------------------------------------------------------------------//
//  Value getting routines

	public int      GetTrainsInStation ()
    	{return m_nTrainsInStation;}
	public int      GetFirstTrainIn ()
    	{return m_nFirstTrainIn;}
	public int      GetSecondTrainIn ()
    	{return m_nSecondTrainIn;}
};
//  End of class StationData
//===========================================================================//


game class MKVIMonorailDemoScenario isclass Scenario
//
//   Class:      MyMKVIMonorailDemo
//   Purpose:    This is the initial Scenario class used to control the MKVIDemo Scenario
//   Copyright (C) 2002 Jack Emmerichs, Milwaukee Wisconsin, 2002.
//   Created by: Jack Emmerichs
//
//   Change History---------
//      06/15/02    Started as a GameObject class.
//      12/02/02    Started a whole new version based on the Scneario class to fix the
//                          problem of running relyably, especially under OpenGL.
{
//  Constant Values

    define public bool  OPEN_DOORS     = true;  // Open the train doors
    define public bool  CLOSE_DOORS    = false; // Close the train doors

    define public int   TRAIN_COUNT    = 3;     // the number of trains in the simulation
    define public int   GREEN          = 0;     // Index of the green train in aTrain
    define public int   BLUE           = 1;     // Index of the blue train in aTrain
    define public int   RED            = 2;     // Index of the red train in aTrain

    define public float SPEED_BRAKE    = -1.00; // speed used to brake for the stations.
    define public float SPEED_STOPPED  = 0.00;  // speed used to stop the train
    define public float SPEED_STATION  = 0.30;  // speed within station areas
    define public float SPEED_FULL     = 1.00;  // speed used to get moving quickly

    define public string NORTH_STATION_NAME    = "LAKESIDE NORTH";
    define public string SOUTH_STATION_NAME    = "LAKESIDE SOUTH";

    public string [] aAnimName  = new string[0];
    public string [] aDoorName  = new string[0];
    
//  Protected Instance Attributes

    Train     [] aTrain         = new Train     [0];    // The array of trains we will be running
    TrainData [] aTrainData     = new TrainData [0];    // The array of TrainData objects describing
                                                        //    the aTrain objects.
    StationData  obStationData  = new StationData ();   // The data describing the demo station

    bool    bInStartup          = true;     // "scenario is starting up" flag
    bool    bTrainsStarted      = false;    // "trains have now been started" flag
    bool    bLogging            = true;     // set true to enable logging <=== set false to disable logging.
    bool    bAnimating          = true;     // set true to animate doors.

    bool    bScenarioDone       = false;    // standard ending flag for scenarios

//  Forward references for Protected methods

    void handleTriggerMessages (Message msg);

    void AnimateTheDoors (Train     theTrain,
                          string    strTrainName,
                          int       nTrainIndex,
                          bool      bIsOpen);

//---------------------------------------------------------------------------//
  bool Load(string data)

// Descr:   Trainz calls Load to load the scenario map, and when the user presses Ctrl-L
//          param data is the save game data if loading a saved game.
// Params:  msg:     The message we are to process.
// Returns: Nothing.

  {
    Interface.Log ("===  Start of Scenario MKVIDemo ===");
    if(data and data.size())
    {
      Interface.Load(data);
    }
//
//      Load the names of the animation movement and mesh files for scripted animation
//
    aAnimName[0] = "DoorAnimGreen.kin";
    aAnimName[1] = "DoorAnimBlue.kin";
    aAnimName[2] = "DoorAnimRed.kin";

    aDoorName[0] = "LeftDoorsGreen_panto.pm";
    aDoorName[1] = "LeftDoorsBlue_panto.pm";
    aDoorName[2] = "LeftDoorsRed_panto.pm";

//      load the map

    if(! World.LoadMap(World.FindKUID("monorail_demo_2")))
    {
      Interface.Log("Error loading scenario map: monorail_demo_2");
      return false;
    }
    Interface.Log ("===  scenario map monorail_demo_2 loaded successfully ===");

//      disable saving on this scenario

    BrowserInterface.SetSaveAllowed(false);

//  Create train consists

    aTrain[0] = World1.CreateTrain("MonorailGreenConsist", "StationNorthMark", true);
    aTrain[1] = World1.CreateTrain("MonorailBlueConsist",  "StationSouthMark", true);
    aTrain[2] = World1.CreateTrain("MonorailRedConsist",   "RedStart",         true);
    aTrainData[0] = new TrainData ();
    aTrainData[1] = new TrainData ();
    aTrainData[2] = new TrainData ();

    aTrainData[0].SetName ("MONORAIL GREEN");
    aTrainData[1].SetName ("MONORAIL BLUE");
    aTrainData[2].SetName ("MONORAIL RED");

//  Set the status and configuration of the train.

    Vehicle[]   vehicles;               // the vehicles that make up a single train
    int         nTrainIndex;            // Index of the train we are building.
    int         nCarIndex;              // Index of the vehicles in a train.
    int         hAnimation;             // Animation handle from added door animation
    for (nTrainIndex = 0; nTrainIndex < TRAIN_COUNT; nTrainIndex++)
    {
        if (bLogging)
            Interface.Log ("---- configuring train " + nTrainIndex);
        aTrain[nTrainIndex].SetPhysicsMode (Train.PHYSICS_DCC);
        aTrain[nTrainIndex].SetAutopilotMode (Train.CONTROL_MANUAL);
        aTrainData[nTrainIndex].SetID (aTrain[nTrainIndex].GetId());
        aTrainData[nTrainIndex].SetIsInStation (false);
        
//  If we are animating the doors, add the doors to the vehicles

        if (bAnimating)
        {
            vehicles = aTrain[nTrainIndex].GetVehicles();
            for (nCarIndex = 0; nCarIndex < vehicles.size(); nCarIndex++)
            {
                hAnimation = vehicles[nCarIndex].AddAnimatedObject("a.pant0", aDoorName[nTrainIndex], aAnimName[nTrainIndex]);
                if (hAnimation == -1)
                {
                    if (bLogging)
                        Interface.Log ("-- AddAnimation for vehicle [" + nCarIndex +
                                       "] failed, animation skipped.");
                }
                else
                {
                    if (bLogging)
                        Interface.Log ("-- Vehicle [" + nCarIndex +
                                       "] has an animation handle of: [" + hAnimation + "].");
                }
            }
        }

//  Set the cameras and log the end of the train setup
        
        World.SetCamera(aTrain[nTrainIndex], World.CAMERA_EXTERNAL);
        Sleep(1.0f);
        World.SetCameraAngle((int)240, (int)-10, (float)25);
        Sleep(1.0f);
        if (bLogging)
            Interface.Log ("-- Finished train " + nTrainIndex);
    }
    
//      Define the message handlers for all three trains.

    for (nTrainIndex = 0; nTrainIndex < TRAIN_COUNT; nTrainIndex++)
    {
        AddHandler(aTrain[nTrainIndex], "Trigger", (string) null, "handleTriggerMessages");
    }
    if (bLogging)
        Interface.Log ("-- End of the Load routine ");

    return true;
  }     // end load



//---------------------------------------------------------------------------//
  void handleTriggerMessages (Message msg)

// Descr:   Handle messages received by the trains.  Messages include those
//          sent by triggers and those we send to the trains.
// Params:  msg:     The message we are to process.
// Returns: Nothing.

  {
    Train       theTrain;               // the train we are controlling
    Trigger     theTrigger;             // the trigger we have just entered
    Vehicle[]   vehicles;               // the vehicles that make up a single train
    int         nTestIndex;             // index used to test which train we are using
    int         nTrainIndex;            // Index of the train we are handling.
    float       fSpeed = 0.0;           // the speed indicated by a speed trigger
    string      strTrainName;           // the name of the train for this message
    string      strTriggerName;         // the name of the trigger for this message
    string      strStationName;         // used to check the name of the station
    string      strWork;                // a general purpose work string

//  Log the message

    if (bLogging)
        Interface.Log ("Demo::handleTriggerMessages msg.major=" + msg.major +
                       " msg.minor=" + msg.minor +
                       " msg.src=" + (cast<GameObject> msg.src).GetName() +
                       " msg.dst=" + (cast<GameObject> msg.dst).GetId());

//  If we are still in startup and not up to setting the tracking cameras yet,
//  it is time to start the trains.

    if (    (bInStartup)
        and (msg.minor != "UseTrackingCamera")
        )
    {
        if (bTrainsStarted)
        {
            if (bLogging)
                Interface.Log ("Starting up and trains already started, exiting message processing.");
        }
        else
        {
            if (bLogging)
                Interface.Log ("Full train appeared at the station, time to start them up.");
                
//  Put the camera on Monorail Green, watch it come out of the station, and
//  then move to the tracking cameras, get the trains moving, and play the horn
//  to start the action.

            World.SetCamera(aTrain[GREEN], World.CAMERA_EXTERNAL);
            World.SetCameraAngle((int)240, (int)-10, (float)25);
            for (nTrainIndex = 0; nTrainIndex < TRAIN_COUNT; nTrainIndex++)
            {
                aTrain[nTrainIndex].SetDCCThrottle (SPEED_FULL);
            }
//      vehicles = aTrain[GREEN].GetVehicles();     // 3D sound debugging code.
//      World.PlaySound ("monorail horn Wave2.wav", 10000.0, 200.0, 6000.0, vehicles[0], "a.limfront");

            World.PlaySound ("monorail horn Wave2.wav", 500.0, 0, 200, null, null);
            World.SetCameraAngle((int)240, (int)-10, (float)25);
            PostMessage(aTrain[GREEN], "Trigger", "UseTrackingCamera", 12);
                
            bTrainsStarted = true;
        }
        return;
    }

//  Handle the various message types.

    if (msg.minor == "Leave")
    {
//  Received a message from a trigger we are leaving.  We don't process the 
//  Leave message, so log this fact, and ignore it.

        if (bLogging)
            Interface.Log ("Leave is not a useful message.  exiting now.");
        return;
    }
//  Received a message to process.  Get the train and the trigger we are using.
//  Also, get the trigger name and the train array index (which provides the train name).

    theTrain   = (cast<Train> msg.dst);
    theTrigger = (cast<Trigger> msg.src);
    strTriggerName = (cast<GameObject> msg.src).GetName();

    for (nTestIndex = 0; nTestIndex < TRAIN_COUNT; nTestIndex++)
    {
        if (theTrain.GetId() == aTrainData[nTestIndex].GetID())
        {
//  Found this train in the array, save the index and get the train name.
        
            nTrainIndex = nTestIndex;
            strTrainName = aTrainData[nTrainIndex].GetName();
            break;
        }
    }

    if (msg.minor == "UseTrackingCamera")
    {
//  Set the view back to the tracking cameras and note that we are no longer
//  in startup (effected by the fist call here only).

        World.SetCamera(theTrain, World.CAMERA_TRACKING);
        bInStartup = false;
        return;
    }
    
    if (msg.minor == "IgnoreStation")
    {
//  Find the train not in the station, and put the tracking cameras on it
//  (watching trains sitting in the station is pretty boreing).

        for (nTestIndex = 0; nTestIndex < TRAIN_COUNT; nTestIndex++)
        {
            if (! aTrainData[nTestIndex].GetIsInStation())  // for the train not in the station
            {
                World.SetCamera(aTrain[nTestIndex], World.CAMERA_TRACKING);
                if (bLogging)
                    Interface.Log ("Ignore station, setting the camer on: " + aTrainData[nTestIndex].GetName());
            }
        }
        return;
    }


    if (msg.minor == "Stopped")
    {
//  A train just stopped moving.  If it is at the station (there should be no other
//  cases), open the doors.  If it is the first train into the station, play the
//  door opening sound file.

        if (aTrainData[nTestIndex].GetIsInStation())
        {
            if (bAnimating)
                AnimateTheDoors (theTrain, strTrainName, nTrainIndex, OPEN_DOORS);
            if (obStationData.GetFirstTrainIn() == nTrainIndex)        // first train to arrive
            {
                if (bLogging)
                    Interface.Log ("Playing Entry .wav file for " + strTrainName + ".");
//                vehicles = theTrain.GetVehicles();     // 3D sound debugging code.
//                World.PlaySound ("OpeningMessage.wav", 10000.0, 200.0, 6000.0, vehicles[0], "a.limfront");
//                World.PlaySound ("Door Open wdw monorail.wav", 0.70, 0, 50, vehicles[0], "a.bog0");
                World.PlaySound ("OpeningMessage.wav", 500.0, 0, 50, null, null);
            }
        }
        return;
    }


    if (msg.minor == "StartTrains")
    {
//  For each train in the station, close its doors, record the fact that it is
//  leaving, start it moving, and mark it no longer in the station.

        for (nTestIndex = 0; nTestIndex < TRAIN_COUNT; nTestIndex++)
        {
            if (aTrainData[nTestIndex].GetIsInStation())
            {
                strTrainName = aTrainData[nTestIndex].GetName();
                if (bAnimating)
                    AnimateTheDoors (aTrain[nTestIndex], strTrainName, nTrainIndex, CLOSE_DOORS);
                if (bLogging)
                    Interface.Log ("Restarting " + strTrainName + " at " +
                                    aTrainData[nTestIndex].GetLastStation() + " now.");
                Interface.Print(strTrainName + " LEAVING " +
                                    aTrainData[nTestIndex].GetLastStation());
                Interface.SetObjective(strTrainName + " now leaving: " +
                                        aTrainData[nTestIndex].GetLastStation(), "");
                Interface.ShowObjective(true);
                aTrain[nTestIndex].SetDCCThrottle (SPEED_FULL);
                aTrainData[nTestIndex].SetIsInStation (false);
            }
        }

//  All trains have now left the station.  Clear the station count, post the message
//  to go back to the tracking cameras in a few seconds, and play the air-horn sound

        obStationData.SetTrainsInStation(0);
        PostMessage(aTrain[GREEN], "Trigger", "UseTrackingCamera", 11);
//      vehicles = aTrain[GREEN].GetVehicles();      // 3D sound debugging code.
//      World.PlaySound ("monorail horn Wave2.wav", 10000.0, 200.0, 6000.0, vehicles[0], "a.limfront");
        World.PlaySound ("monorail horn Wave2.wav", 500.0, 0, 200, null, null);
        if (bLogging)
            Interface.Log ("Playing monorail horn Wave2.wav.");
        return;
    }
    

    else if (msg.minor == "LeaveCabAtStation")  // change back from the cab view
    {

//  Move the camera back to the second train into the station.

        if (bLogging)
            Interface.Log ("Handling LeaveCabAtStation message.");
        World.SetCamera(aTrain[obStationData.GetSecondTrainIn()], World.CAMERA_EXTERNAL);
        World.SetCameraAngle((int)240, (int)-10, (float)25);
    }



    if (msg.minor == "Enter")
    {

//  We just entered a trigger.  Process this message depending on the trigger name

        strStationName = strTriggerName;
        Str.Right (strStationName, 5);


        if (strTriggerName[0,5] == "Speed")         // Entered a speed control trigger
        {
            fSpeed = Str.ToFloat(strTriggerName[6,]) / 100;
            theTrain.SetDCCThrottle (fSpeed);
            if (bLogging)
                Interface.Log ("For trigger name [" + strTriggerName + "], Setting " + strTrainName + " speed to: " + fSpeed);
        }

        else if (strTriggerName == "StartInCab")    // Start viewing from inside the cab for Green
        {
            if (nTrainIndex == GREEN)
                World.SetCamera(theTrain, World.CAMERA_INTERNAL);
        }

        else if (strTriggerName == "EndInCab")      // End viewing from inside the cab for Green
        {
            if (nTrainIndex == GREEN)
                World.SetCamera(theTrain, World.CAMERA_TRACKING);
        }

        else if (strTriggerName[0,5] == "Brake")
        {
//  Breaking for an upcoming station.  Log that, and brake

            if (obStationData.GetTrainsInStation() < 1)                 // first train to arrive
            {
                obStationData.SetTrainsInStation(1);
                obStationData.SetFirstTrainIn(nTrainIndex);
                if (nTrainIndex == GREEN)                               // if green, drive into the station.
                    World.SetCamera(theTrain, World.CAMERA_INTERNAL);
            }
            else                                                        // second train to arrive
            {
                obStationData.SetTrainsInStation(2);
                obStationData.SetSecondTrainIn(nTrainIndex);
            }
            if (bLogging)
                Interface.Log ("Breaking " + strTrainName + " now.");
            theTrain.SetDCCThrottle (SPEED_BRAKE);
        }

        else if (strTriggerName[0,8] == "Approach")
        {
//  Approaching a station.  Log it and slow the train down.  If this is
//  the second train to arrive, put the camera on the train.

            aTrainData[nTrainIndex].SetIsInStation (true);
            if (strStationName == "North")
            {
                strTriggerName = NORTH_STATION_NAME;
            }
            else
            {
                strTriggerName = SOUTH_STATION_NAME;
            }
            
            if (    (obStationData.GetSecondTrainIn() == nTrainIndex)   // second train to arrive
                and (obStationData.GetFirstTrainIn() != GREEN))         // and green is not "driving"
            {
                World.SetCamera(theTrain, World.CAMERA_EXTERNAL);       // follow this train with the camera
                World.SetCameraAngle((int)240, (int)-10, (float)25);
            }

            if (bLogging)
                Interface.Log ("Slowing " + strTrainName + " at " + strTriggerName + " now.");
            Interface.Print(strTrainName + " APPROACHING " + strTriggerName);
            theTrain.SetDCCThrottle (SPEED_STATION);
        }


        else if (strTriggerName[0,7] == "Station")
        {
//  Arrived at a station.  Log it, and stop the train.  Also, if this is the last
//  (second) train to arrive, post a message to ignore the station after another
//  few seconds as it's pretty boring.  It doesn't matter who you send it to as
//  we'll simply catch it and then go look at the proper train.

            if (strStationName == "North")
            {
                strTriggerName = NORTH_STATION_NAME;
            }
            else
            {
                strTriggerName = SOUTH_STATION_NAME;
            }
            if (obStationData.GetSecondTrainIn() == nTrainIndex)        // second train to arrive
            {
                PostMessage(theTrain, "Trigger", "IgnoreStation", 9);
                if (bLogging)
                    Interface.Log ("Posting IgnoreStation message.");
            }
            else
            {
                if (nTrainIndex == GREEN)                               // if green, about to leave driver seat
                {
                    if (bLogging)
                        Interface.Log ("Posting LeaveCabAtStation message.");
                    PostMessage(aTrain[GREEN], "Trigger", "LeaveCabAtStation", 2);
                }
            }

            if (bLogging)
                Interface.Log ("Stopping " + strTrainName + " at " + strTriggerName + " now.");
            Interface.Print(strTrainName + " NOW AT " + strTriggerName);
            Interface.SetObjective(strTrainName + " has arrived at: " + strTriggerName, "");
            Interface.ShowObjective(true);
            theTrain.SetDCCThrottle (SPEED_STOPPED);

//  Set the current stop for the train and indicate it is now in the station.

            aTrainData[nTrainIndex].SetLastStation (strTriggerName);
        }


        else if (strTriggerName == "EmptyStation")  // leaving the station
        {
            Train   WatchedTrain;       // the train to watch as it leaves
            string  strTrainName;       // the name of the train being restarted

//  The free-running train has tripped the trigger to restart the trains in the
//  station.  Start the Close doors sound and put the camera on the proper train.

            if (bLogging)
                Interface.Log ("Playing Door Close jack wagner monorail.wav.");

            if (obStationData.GetFirstTrainIn() == GREEN)   // watch green train
                WatchedTrain = aTrain[GREEN];
            else                                            // or second train in
                WatchedTrain = aTrain[obStationData.GetSecondTrainIn()];
//            vehicles = WatchedTrain.GetVehicles();        // 3D sound debugging code.
//            World.PlaySound ("ClosingMessage.wav", 10000.0, 200.0, 6000.0, vehicles[0], "a.limfront");
            World.PlaySound ("ClosingMessage.wav", 500.0, 0, 50, null, null);
            World.SetCamera(WatchedTrain, World.CAMERA_EXTERNAL);
            World.SetCameraAngle((int)240, (int)-10, (float)25);
            PostMessage(aTrain[GREEN], "Trigger", "StartTrains", 2.5f);
        }
    }
  }     // end handleTriggerMessages


//---------------------------------------------------------------------------//
  void AnimateTheDoors (Train   theTrain,
                        string  strTrainName,
                        int     nTrainIndex,
                        bool    bIsOpen)

// Descr:   Open or close all the doors of a train.
// Params:  theTrain        // The train whose doors we are going to animate
//          strTrainName    // The name of the train to animate
//          bIsOpen         // Indicates whether to open or close the doors.
// Returns: Nothing.

  {
    Vehicle[]   vehicles = theTrain.GetVehicles();
    string      m_strMsg = "close";
    int         nCarIndex;
    int         hAnimation;
    if (bIsOpen)
        m_strMsg = "open";

//  Log the animation we are about to try

    Interface.Log ("Try to [" + m_strMsg + "]the doors on [" + strTrainName + "].");
    for (nCarIndex = 0; nCarIndex < vehicles.size(); nCarIndex++)
    {
        hAnimation = vehicles[nCarIndex].GetAnimatedObject("a.pant0");
        if (hAnimation == -1)
        {
            if (bLogging)
                Interface.Log ("-- GetAnimation for vehicle [" + nCarIndex +
                               "] has no animation handle: [" + hAnimation + "], trying AddAnimatedObject.");
            hAnimation = vehicles[nCarIndex].AddAnimatedObject("a.pant0", aDoorName[nTrainIndex], aAnimName[nTrainIndex]);
            if (hAnimation == -1)
            {
                if (bLogging)
                    Interface.Log ("-- AddAnimation for vehicle [" + nCarIndex +
                                   "] has no animation handle: [" + hAnimation + "], animation skipped.");
                return;
            }
        }
        if (bLogging)
            Interface.Log ("-- Vehicle [" + nCarIndex +
                           "] has an animation handle of: [" + hAnimation + "].");
        vehicles[nCarIndex].PlayDirection(hAnimation, bIsOpen);
    }
  }

//---------------------------------------------------------------------------//
//
//  The following routines are created by the standard initial scenario template
//
//---------------------------------------------------------------------------//


  //
  // Save will be called by Trainz when the user presses Ctrl-S.
  // return the save game string, such that load will be able to restor the save game
  // from the last save check point.
  //  
  string Save()
  {
    return Interface.Save ();
  }

  //
  // TrainDerailed will be called by Trainz when a train derails
  //
  void TrainDerailed (int trainId)
  {
    if (! bScenarioDone)
    {
      World.EndScenario (10);
      bScenarioDone = true;
    }
  }

  //
  // TrainCollided will be called by Trainz when a train collides
  //
  void TrainCollided (int trainId)
  {
    if (! bScenarioDone)
    {
      World.EndScenario (10);
      bScenarioDone = true;
    }
  }

  //
  // TrainSpeedingFine() is called by Trainz every second your trains speed exceeds the floating limit
  //
  void TrainSpeedingFine ()
  {
    //Interface.AdjustScore(-10);  
  }

  //
  // TrainBadCouple() is called by Trainz when vehicles couple greater than 8KPH.
  //
  void TrainBadCouple (int vehicleId)
  {
    //Interface.AdjustScore(-200);  
  }

  //
  //
  // main thread
  // brief main is executed automatically after Load() is called.  edit
  // main to contain your scenarios gameplay.
  //
  //    Note: There is no actual gameplay in this scenario; the trains essentially
  //            play by themselves!
  //
  //

  thread void main (void)
  {
    // Start the monitor thread to monitor speeding, derailing etc.
    Monitor();

    bScenarioDone = true;
  }
};
//---------------------------------------------------------------------------//

